@@ -17,18 +17,21 @@
  * @brief Exports vtkContext2D scenes to PDF.
  *
  * This exporter draws context2D scenes into a PDF file.
+ *
+ * If ActiveRenderer is specified then it exports contents of
+ * ActiveRenderer. Otherwise it exports contents of all renderers.
  */
 
 #ifndef vtkPDFExporter_h
 #define vtkPDFExporter_h
 
-#include "vtkIOExportModule.h" // For export macro
+#include "vtkIOExportPDFModule.h" // For export macro
 #include "vtkExporter.h"
 
 class vtkContextActor;
 class vtkRenderer;
 
-class VTKIOEXPORT_EXPORT vtkPDFExporter: public vtkExporter
+class VTKIOEXPORTPDF_EXPORT vtkPDFExporter: public vtkExporter
 {
 public:
   static vtkPDFExporter* New();
